/**
 * COMS E6181: Advanced Internet Services (Fall 2011)
 * pgs2111 at columbia.edu
 */
package edu.columbia.cs.dyswis.module_mcast_rtp;

import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import net.sourceforge.jpcap.net.Packet;
import edu.columbia.cs.dyswis.common.baseclass.DetectInterface;
import edu.columbia.cs.dyswis.common.baseclass.DysDetect;

/**
 * Detector and Session History maintainer for multicast RTP packets.
 * <p>
 * Listens for multicast RTP packets. Maintains a history of all multicast RTP
 * sessions. Raises faults if sequence number gaps are observed.
 * 
 * @author phil
 */
public class McastRtpDetect extends DysDetect implements
        DetectInterface {

    /** for log messages */
    private static Logger logger = Logger.getLogger("McastRtpDetect");

    /**
     * Map of session identifiers to detected sessions.
     * 
     * @see McastRtpSession#makeSessionIdentifier(McastRtpPacket)
     */
    private Map<String, McastRtpSession> mcastRtpSessionMap = null;

    /**
     * Default pcap filter. Looks for multicast destination address and RTP
     * version 2.
     */
    private static final String defaultPcapFilter =
            "net 224.0.0.0/4 and (udp[8:1]&0x80)==0x80";

    /**
     * Name of protocol
     */
    private static final String protocol = "MCAST_RTP";

    /**
     * Default constructor
     */
    public McastRtpDetect() {
        super();

        filter = defaultPcapFilter;
        mcastRtpSessionMap =
                Collections
                        .synchronizedMap(new LinkedHashMap<String, McastRtpSession>());

        logger.info("McastRtpDetect starting");
    }

    /**
     * Handles a received packet. If it is a Multicast RTP packet, finds an
     * existing session or creates a new one. Then notifies the session that a
     * packet has arrived.
     * 
     * @param packet
     *            the Packet that just arrived.
     * @see McastRtpSession#packetArrived(Packet)
     */
    @Override
    public void packetArrived(Packet packet) {

        // logger.debug("packetArrived: " + packet.toString());

        /*
         * this is a weird one. why are we getting packets that don't match our
         * filter?
         */
        if (!McastRtpPacket.checkPacket(packet)) {
            // logger.debug("packetArrived: checkPacket failed");
            return;
        }
        McastRtpPacket mcastRtpPacket = null;

        /*
         * Even if checkPacket succeeds, this still might not be a valid packet
         * .. we won't know for sure until we parse it.
         */
        try {
            mcastRtpPacket = new McastRtpPacket(packet);
        } catch (IllegalArgumentException iae) {
            logger.debug(iae);
            return;
        }

        /* moved by phil - don't count it unless it is valid */
        packetCount++;

        /*
         * Try to match the packet against an existing session, or create a new
         * session if necessary.
         */
        String mcastRtpSessionId =
                McastRtpSession.makeSessionIdentifier(mcastRtpPacket);

        if (mcastRtpSessionMap.containsKey(mcastRtpSessionId)) {
            mcastRtpSessionMap.get(mcastRtpSessionId)
                    .mcastRtpPacketArrived(mcastRtpPacket);
        } else {
            mcastRtpSessionMap.put(mcastRtpSessionId,
                    new McastRtpSession(mcastRtpPacket));

            // run the RPF check now. we'll probably need it.
            Thread rpfThread =
                    new McastRtpReversePathDiscoveryThread(
                            mcastRtpPacket.getSourceAddress());
            rpfThread.start();
        }

    }

    /**
     * Retrieve a session from the session history
     * 
     * @param sessionId
     *            the session identifier
     * @return the associated session, or null if not found
     */
    public McastRtpSession getSession(String sessionId) {
        return mcastRtpSessionMap.get(sessionId);
    }

    /**
     * Retrieve the number of multicast RTP sessions
     * 
     * @return the number of multicast RTP sessions
     */
    @Override
    public int getSessionSize() {
        return mcastRtpSessionMap.size();
    }

    /**
     * Required override. Not used.
     */
    @Override
    public void loadConfig() {
        // TODO Auto-generated method stub
    }

    /**
     * Get the total number of McastRtpPackets for statistics collection
     * 
     * @return the number of McastRtpPackets detected.
     */
    @Override
    public int getPacketCount() {
        return packetCount;
    }

    /**
     * Get the name of the protocol.
     * 
     * @return the number of this protocol.
     */
    @Override
    public String getProtocolName() {
        return protocol;
    }

}
