/**
 * COMS E6181: Advanced Internet Services (Fall 2011)
 * pgs2111 at columbia.edu
 */
package edu.columbia.cs.dyswis.module_mcast_rtp;

import java.text.SimpleDateFormat;
import java.util.HashMap;

import org.apache.log4j.Logger;

import edu.columbia.cs.dyswis.common.baseclass.DysFault;
import edu.columbia.cs.dyswis.common.baseclass.DysPacket;
import edu.columbia.cs.dyswis.core.diagnosis.faultoperator.DysFaultOperator;

/**
 * This class represents a fault while receiving a multicast RTP stream.
 * <p>
 * Currently, the only type of fault defined is a sequence number gap.
 * 
 * @author phil
 */
public class McastRtpFault extends DysFault {

    /**	serialVersionUID required since DysFault is serializable */
    private static final long serialVersionUID = -5583071235806013543L;

    /** RTP sequence number of the next packet we are expecting to receive */
    private int expected;

    /** RTP sequence number of the packet we received */
    private int received;

    /** the time the of the last packet we received before the gap */
    private long gapStart;

    /** the time of the first packet we received after the gap */
    private long gapEnd;

    /** the McastRtpSession identifier */
    private String sessionId;

    /** for log messages */
    private static Logger logger = Logger.getLogger("McastRtpFault");

    /**
     * Generates a fault from a multicast rtp packet.
     * <p>
     * Only populates the information about the session, but does not specify
     * any details about the fault itself.
     * 
     * @param packet
     *            the packet that triggered the fault
     */
    public McastRtpFault(DysPacket packet) {
        super(packet);

        assert (packet instanceof McastRtpPacket);

        protocolName = "MCAST_RTP";
        hostAddr = ((McastRtpPacket) packet).getSourceAddress();
        hostPort = ((McastRtpPacket) packet).getSourcePort();
        myAddr = ((McastRtpPacket) packet).getDestinationAddress();
        myPort = ((McastRtpPacket) packet).getDestinationPort();
        sessionId =
                McastRtpSession
                        .makeSessionIdentifier((McastRtpPacket) packet);
        type = DysFaultOperator.MCAST_RTP;
    }

    /**
     * Store information about the fault in a hash map. The hashmap is then sent
     * to the remote probe receiver.
     * 
     * @see edu.columbia.cs.dyswis.common.baseclass.DysFault#makeHashMap()
     */
    @Override
    public HashMap<?, ?> makeHashMap() {
        logger.debug("creating hash map");
        HashMap<String, String> hm = new HashMap<String, String>();
        hm.put("FAULT", this.toString());
        hm.put("SOURCE", this.getHostAddr());
        hm.put("SOURCE_PORT", Integer.toString(this.getHostPort()));
        hm.put("DEST", this.getMyAddr());
        hm.put("DEST_PORT", Integer.toString(this.getMyPort()));
        hm.put("SESSIONID", sessionId);
        hm.put("GAP_START", Long.toString(gapStart));
        hm.put("GAP_END", Long.toString(gapEnd));
        return hm;
    }

    /**
     * Sets the sequence number that was expected
     * <p>
     * Called by McastRtpDetect when creating the fault
     * 
     * @param i
     *            the expected sequence number.
     */
    public void setExpected(int i) {
        expected = i;
    }

    /**
     * Sets the sequence number that was received
     * <p>
     * Called by McastRtpDetect when creating the fault
     * 
     * @param i
     *            the received sequence number.
     */
    public void setReceived(int i) {
        received = i;
    }

    /**
     * Retrieve the expected sequence number from a fault
     * 
     * @return the expected sequence number
     */
    public int getExpected() {
        return expected;
    }

    /**
     * Retrieve the received sequence number from a fault
     * 
     * @return the received sequence number
     */
    public int getReceived() {
        return received;
    }

    /**
     * Display information about the fault
     * 
     * @return a String representation of the fault.
     */
    @Override
    public String toString() {
        McastRtpPacket mcastRtpPacket = (McastRtpPacket) this.packet;

        String result =
                McastRtpSession.makeSessionIdentifier(mcastRtpPacket);
        result += " Expected seq: " + getExpected();
        result += " Received seq: " + getReceived();

        return result;
    }

    /**
     * Set the start and end time of the gap
     * <p>
     * Time values are in the format of System.currentTimeMillis()
     * 
     * @param start
     *            the start time
     * @param end
     *            the end time
     */
    public void setGap(long start, long end) {

        SimpleDateFormat fmt =
                new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        logger.debug("gap start: " + fmt.format(start));
        logger.debug("gap end: " + fmt.format(end));

        gapStart = start;
        gapEnd = end;
    }

    /**
     * @return the sessionId
     */
    public String getSessionId() {
        return sessionId;
    }

    /**
     * @return the gapStart
     */
    public long getGapStart() {
        return gapStart;
    }

    /**
     * @return the gapEnd
     */
    public long getGapEnd() {
        return gapEnd;
    }

}