/**
 * COMS E6181: Advanced Internet Services (Fall 2011)
 * pgs2111 at columbia.edu
 */
package edu.columbia.cs.dyswis.module_mcast_rtp;

import java.io.Serializable;
import java.nio.ByteBuffer;

import org.apache.log4j.Logger;

import net.sourceforge.jpcap.net.Packet;
import net.sourceforge.jpcap.net.UDPPacket;
import edu.columbia.cs.dyswis.common.baseclass.DysPacket;

/**
 * This class represents a multicast RTP packet.
 * <p>
 * Specifically, this is an RTP version 2 packet destined for a multicast group
 * address.
 * 
 * @author phil
 */
public class McastRtpPacket extends DysPacket implements Serializable {

    /** serialVersionUID so we can serialize the McastRtpPacket */
    private static final long serialVersionUID = 7184723097372608037L;

    /** for log messages */
    private static Logger logger = Logger.getLogger("McastRtpPacket");

    /** rtp header field */
    private byte version;
    /** rtp header field */
    private byte p;
    /** rtp header field */
    private byte x;
    /** rtp header field */
    private byte cc;
    /** rtp header field */
    private byte m;
    /** rtp header field */
    private byte pt;
    /** rtp header field */
    private int seq; /* 16 bits, but java has no unsigned short :( */
    /** rtp header field */
    private long ts; /* 32 bits, but java has no unsigned int :( */
    /** rtp header field */
    private long ssrc; /* 32 bits, but java has no unsigned int :( */
    /** rtp header field */
    private int csrc[];
    /** rtp header field */
    private short ext_type;
    /** rtp header field */
    private short ext_len;
    /** rtp header field */
    private int ext_data[];

    /**
     * arrival time of the packet (millis)
     */
    private long arrivalTime;

    /**
     * @param packet
     */
    public McastRtpPacket(Packet packet) {

        super(packet);
        arrivalTime = System.currentTimeMillis();

        transProtocol = P_UDP;
        applProtocol = P_MCAST_RTP;
        protocolName = "MCAST_RTP";

        if (!__parse(packet)) {
            throw new IllegalArgumentException(packet.toString());
        }

        // logger.debug("McastRtpPacket: " + udpPacket.toString());
    }

    /**
     * Mandatory override. Not really applicable for McastRtpPacket - always
     * returns false.
     * 
     * @return false
     */
    @Override
    public boolean containsErrorResponse() {
        return false;
    }

    /**
     * Quick check to see if a packet meets the criteria for a multicast RTP
     * packet.
     * <p>
     * Needs to be: <br>
     * 1. UDP <br>
     * 2. Have a class D destination address <br>
     * 3. Be RTP version 2 <br>
     * 4. Not an RTCP packet <br>
     * <p>
     * Not a bad idea to call this before handing a packet to the constructor :)
     * 
     * @param packet
     *            the packet to examine
     * @return false for an invalid packet, true if it might be valid
     */
    public static boolean checkPacket(Packet packet) {
        /* make sure it's udp */
        if (!(packet instanceof UDPPacket)) {
            // logger.debug("checkPacket: NOT UDP: " + packet.toString());
            return false;
        }
        UDPPacket udpPacket = (UDPPacket) packet;

        /* check the first octet to make sure it is a class D */
        int b = udpPacket.getDestinationAddressBytes()[0] & 0xff;
        if (b < 224 || b > 239) {
            // logger.debug("checkPacket: NOT Multicast: " + packet.toString());
            return false;
        }

        /* check the length of the packet */
        if (udpPacket.getLength() < 12) {
            // logger.debug("checkPacket: NOT RTP version 2 (too short): " +
            // packet.toString());
            return false;
        }

        /* check the RTP version */
        b = udpPacket.getData()[0];
        if ((b & 0xc0) != 0x80) {
            // logger.debug("checkPacket: NOT RTP version 2 (incorrect version): "
            // + packet.toString());
            return false;
        }

        /* check if it might be RTCP */
        b = udpPacket.getData()[1];
        if ((b & 0xfe) == 0xc8) {
            // logger.debug("checkPacket: Looks like RTCP: " +
            // packet.toString());
            return false;
        }

        /* well, if it matches this far, that's a pretty good sign .. */
        return true;
    }

    /**
     * Actually parse the packet. Called by the constructor.
     * 
     * @param packet
     *            the packet to parse
     * @return true if parse is successful, false otherwise
     */
    public boolean __parse(Packet packet) {

        if (!checkPacket(packet)) {
            return false;
        }

        ByteBuffer buf = ByteBuffer.wrap(udpPacket.getUDPData());
        int len = buf.remaining();
        if (len < 12) {
            logger.error("packet too short");
            return false;
        }

        int word = buf.getInt();
        version = (byte) ((word >> 30) & 0x03);
        p = (byte) ((word >> 29) & 0x01);
        x = (byte) ((word >> 28) & 0x01);
        cc = (byte) ((word >> 24) & 0x0f);
        m = (byte) ((word >> 23) & 0x01);
        pt = (byte) ((word >> 16) & 0x7f);
        seq = word & 0xffff;
        ts = (buf.getInt() & 0xffffffffL);
        ssrc = (buf.getInt() & 0xffffffffL);

        /*
         * check the version first (checkPacket should have caught this)
         */
        if (version != 2) {
            logger.error("RTP version incorrect");
            return false;
        }

        /*
         * Make sure this isn't RTCP. Check the payload type, and make sure that
         * it is neither RTCP_SR (200 = 0xc8) nor RTCP_RR (201 = 0xc9) In RTP,
         * the second byte consists of unsigned int m:1 unsigned int pt:7 In
         * RTCP, the second byte is just unsigned int pt:8
         */
        if (m == 0x1 && (pt == 0x48 || pt == 0x49)) {
            logger.error("M: " + m + " PT: " + pt
                    + ": possible RTCP packet?");
            return false;
        }

        /* any csrcs? */
        if (cc > 16) {
            logger.error("CSRC count invalid");
            return false;
        }

        int hlen = 12 + 4 * cc;
        if (len < hlen) {
            logger.error("packet too short");
            return false;
        }

        if (cc > 0) {
            csrc = new int[cc];
            for (int i = 0; i < cc; i++) {
                csrc[i] = buf.getInt();
            }
        }

        /* header extensions? */
        if (x == 1) {

            if (len < hlen + 4) {
                logger.error("packet too short");
                return false;
            }

            ext_type = buf.getShort();
            ext_len = buf.getShort();

            if (len < hlen + 4 + ext_len * 4) {
                logger.error("packet too short");
                return false;
            }

            if (ext_len > 0) {
                ext_data = new int[ext_len];
                for (int i = 0; i < ext_len; i++) {
                    ext_data[i] = buf.getInt();
                }
            }
        }

        return true;
    }

    /**
     * Mandatory override. Does nothing - the packet should already be parsed by
     * the constructor.
     */
    @Override
    public void parse() {
        /*
         * do nothing. the packet should already be parsed.
         */
        return;
    }

    /**
     * @return the src_addr
     */
    public String getSourceAddress() {

        return udpPacket.getSourceAddress();
    }

    /**
     * @return the src_port
     */
    public int getSourcePort() {

        return udpPacket.getSourcePort();
    }

    /**
     * @return the dst_addr
     */
    public String getDestinationAddress() {

        return udpPacket.getDestinationAddress();
    }

    /**
     * @return the dst_port
     */
    public int getDestinationPort() {

        return udpPacket.getDestinationPort();
    }

    /**
     * @return the seq
     */
    public int getSeq() {

        return seq;
    }

    /**
     * @return the ssrc
     */
    public long getSsrc() {

        return ssrc;
    }

    /**
     * @return a string representation of the packet.
     */
    @Override
    public String toString() {

        StringBuffer buffer = new StringBuffer();

        buffer.append("[");
        buffer.append("McastRtpPacket");
        buffer.append(": ");
        buffer.append("v=" + version);
        buffer.append(":p=" + p);
        buffer.append(":x=" + x);
        buffer.append(":cc=" + cc);
        buffer.append(":m=" + m);
        buffer.append(":pt=" + pt);
        buffer.append(":seq=" + seq);
        buffer.append(":ts=" + ts);
        buffer.append(":ssrc=" + ssrc);

        for (int i = 0; i < cc; i++) {
            buffer.append(":csrc[" + i + "]=" + csrc[i]);
        }

        if (x == 1) {
            buffer.append(":ext_type=" + ext_type);
            buffer.append(":ext_len=" + ext_len);
            for (int i = 0; i < ext_len; i++) {
                buffer.append(":ext_data[" + i + "]=" + ext_data[i]);
            }
        }

        buffer.append("]");
        return buffer.toString();
    }

    /**
     * @return the arrivalTime
     */
    public long getArrivalTime() {
        return arrivalTime;
    }

    /**
     * @param arrivalTime
     *            the arrivalTime to set
     */
    public void setArrivalTime(long arrivalTime) {
        this.arrivalTime = arrivalTime;
    }

}
