/**
 * COMS E6181: Advanced Internet Services (Fall 2011)
 * pgs2111 at columbia.edu
 */
package edu.columbia.cs.dyswis.module_mcast_rtp;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

import edu.columbia.cs.dyswis.common.baseclass.DysProbe;
import edu.columbia.cs.dyswis.common.util.Functions;

/**
 * Find the path towards the multicast source IP. (by running a traceroute)
 * 
 * @author phil
 */
public class McastRtpReversePathDiscovery implements DysProbe {

    /** for log messages */
    private static Logger logger = Logger
            .getLogger("McastRtpReversePathDiscovery");

    /**
     * traceroute command for Windows
     * <p>
     * -d no names resolution<br>
     * -w 1000 1 second timeout<br>
     * -h 30 maximum of 30 hops
     */
    private static final String tracerouteCmdWindows =
            "tracert -d -w 1000 -h 30";
    /**
     * traceroute command for OSX/Linux/other platforms
     * <p>
     * -n no names resolution<br>
     * -w 1 1 second timeout<br>
     * -m 30 maximum of 30 hops
     */
    private static final String tracerouteCmdGeneric =
            "traceroute -n -w 1 -m 30";

    /**
     * Maps IP address (of multicast source) => traceroute hops towards the
     * source (hops are separated by ' ')
     */
    private static Map<String, String> sourceMap =
            new ConcurrentHashMap<String, String>();

    /**
     * Perform a traceroute towards the multicast source. Store the nodes in a
     * space-separated string.
     * 
     * @param faultMap
     *            either an McastRtpFault, or a HashMap containing a mapping
     *            from key "SOURCE" to a String with the multicast source IP
     */
    @Override
    public Object probe(Object faultMap) {

        logger.debug("probe: " + faultMap);

        /*
         * if called directly from the GUI with an actual McastRtpFault, probe
         * towards the source and display messages on the GUI
         */
        if (faultMap instanceof McastRtpFault) {
            McastRtpFault fault = (McastRtpFault) faultMap;
            return probe(fault.getHostAddr(), true);
        }
        /*
         * if called by a remote probe, probe towards the source and don't
         * display messages
         */
        else if (faultMap instanceof HashMap<?, ?>) {
            String mcastRtpSource =
                    (String) ((HashMap<?, ?>) faultMap).get("SOURCE");
            return probe(mcastRtpSource, false);
        } else {
            logger.debug("Unknown object: " + faultMap);
            return null;
        }
    }

    /**
     * Runs the traceroute, displaying messages if desired.
     * <p>
     * Cache the results, since traceroute can be slow if you have lots of
     * intermediate hops that time out.
     * 
     * @param mcastRtpSource
     * @param displayGuiMessages
     * @return a String containing space-separated hops
     */
    public static String probe(String mcastRtpSource,
            boolean displayGuiMessages) {
        logger.debug("RPF check towards: " + mcastRtpSource);
        if (displayGuiMessages) {
            Activator.getDyswis().addDiagnosisMessageContent(
                    "Traceroute to " + mcastRtpSource, 0);
        }

        /* check if we have seen this source before */
        if (sourceMap.containsKey(mcastRtpSource)) {

            logger.debug("Already seen source: " + mcastRtpSource);
            return getCachedResult(mcastRtpSource, displayGuiMessages);
        }

        /* run the traceroute and store the result */
        StringBuilder hops = new StringBuilder();

        Runtime rt = Runtime.getRuntime();

        String tracerouteCmd;
        if (Functions.getMyOSType() == Functions.OS_WINDOWS) {
            tracerouteCmd = tracerouteCmdWindows;
        } else {
            tracerouteCmd = tracerouteCmdGeneric;
        }

        try {
            Process process =
                    rt.exec(tracerouteCmd + " " + mcastRtpSource);
            BufferedReader bf =
                    new BufferedReader(new InputStreamReader(
                            process.getInputStream()));
            String line = null;
            String regexp =
                    "(\\d+)\\b.*?\\b(\\d+\\.\\d+\\.\\d+\\.\\d+)\\b";

            Pattern p = Pattern.compile(regexp);
            logger.debug("pattern: " + p.toString());

            while ((line = bf.readLine()) != null) {
                logger.debug(line);
                Matcher m = p.matcher(line);

                if (m.find()) {
                    int hop = Integer.parseInt(m.group(1));
                    String ip = m.group(2);
                    logger.debug("hop: " + hop + " ip: " + ip);
                    if (displayGuiMessages) {
                        Activator
                                .getDyswis()
                                .addDiagnosisMessageDescribe(
                                        "Hop: " + hop + " IP: " + ip, 0);
                    }
                    hops.append(ip + " ");
                }
            }
            logger.debug("traceroute complete");
        } catch (IOException e) {
            System.out.println(e.getStackTrace());
        }

        String result = hops.toString();
        sourceMap.put(mcastRtpSource, result);
        logger.debug("returning: result: " + result);
        return result;
    }

    /**
     * Retrieves the cached traceroute results. May be called locally or
     * remotely.
     * 
     * @param mcastRtpSource
     *            the source IP address
     * @param displayGuiMessages
     *            if true, update the GUI
     * @return the list of hops separates by spaces
     */
    private static String getCachedResult(String mcastRtpSource,
            boolean displayGuiMessages) {

        /* retrieve the cached source */
        String result = sourceMap.get(mcastRtpSource);

        /* update the GUI if necessary */
        if (displayGuiMessages) {
            String[] hopArray = result.split(" ");
            for (int i = 0; i < hopArray.length; i++) {
                Activator.getDyswis().addDiagnosisMessageDescribe(
                        "Cached Hop: IP: " + hopArray[i], 0);
            }
        }

        return result;
    }

    /**
     * Display a message that the traceroute is complete.
     * <p>
     * Doesn't need to do too much, since the probe itself updates the GUI as it
     * goes along.
     * 
     * @param request
     *            ignored
     * @param result
     *            ignored
     */
    @Override
    public void displayResult(Object request, Object result) {
        Activator.getDyswis().addDiagnosisMessageDescribe(
                "Traceroute complete", 0);
        Activator.getDyswis().addDiagnosisMessageDescribe(
                result.toString(), 0);
    }

}
