/**
 * COMS E6181: Advanced Internet Services (Fall 2011)
 * pgs2111 at columbia.edu
 */
package edu.columbia.cs.dyswis.module_mcast_rtp;

import org.apache.log4j.Logger;

import edu.columbia.cs.dyswis.common.baseclass.DysSession;

/**
 * An McastRtpSession represents a unique multicast stream.
 * <p>
 * The stream is uniquely identified by the 4-tuple of source IP address, source
 * port, destination multicast group address, destination port.
 * <p>
 * McastRtpSessions are created by McastRtpDetect when an McastRtpPacket is
 * received that does not match an existing session.
 * 
 * @author phil
 */
public class McastRtpSession extends DysSession {

    /** for log messages */
    private static Logger logger = Logger.getLogger("McastRtpSession");

    /**
     * The SSRC associated with this session
     */
    private long ssrc;

    /**
     * The last sequence number that was received for this session
     */
    private int lastSeqReceived;

    /**
     * The next sequence number that is expected for this session
     */
    private int nextSeqExpected;

    /**
     * Time that the session was started (in millis)
     */
    private long sessionStartTime;

    /**
     * Time that the last packet was received
     */
    private long lastPacketTime;

    /**
     * Create a new Multicast RTP session with an identifier that matches the
     * received packet.
     * 
     * @param mcastRtpPacket
     *            the packet used to build the session identifier
     */
    public McastRtpSession(McastRtpPacket mcastRtpPacket) {
        super();
        sessionStartTime = mcastRtpPacket.getArrivalTime();

        ssrc = mcastRtpPacket.getSsrc();
        lastSeqReceived = mcastRtpPacket.getSeq();
        nextSeqExpected = (lastSeqReceived + 1) & 0xffff;
        logger.info("NEW SESSION: "
                + makeSessionIdentifier(mcastRtpPacket));
    }

    /**
     * Generate a session identifier from a Multicast RTP packet
     * 
     * @param mcastRtpPacket
     *            the multicast RTP packet
     * @return a String that can be used as a session identifier
     */
    public static String makeSessionIdentifier(
            McastRtpPacket mcastRtpPacket) {

        if (mcastRtpPacket == null) {
            return "INVALID MCAST_RTP_PACKET";
        }

        return makeSessionIdentifier(mcastRtpPacket.getSourceAddress(),
                Integer.toString(mcastRtpPacket.getSourcePort()),
                mcastRtpPacket.getDestinationAddress(),
                Integer.toString(mcastRtpPacket.getDestinationPort()));
    }

    /**
     * Generate a session identifier from the required identifiers
     * 
     * @param srcAddr
     *            source IP address
     * @param srcPort
     *            source port
     * @param dstAddr
     *            destination (multicast) IP address
     * @param dstPort
     *            destination port
     * @return a String that can be used as a session identifier
     */
    public static String makeSessionIdentifier(String srcAddr,
            String srcPort, String dstAddr, String dstPort) {
        StringBuffer buffer = new StringBuffer();

        buffer.append(srcAddr);
        buffer.append("/");
        buffer.append(srcPort);
        buffer.append("->");
        buffer.append(dstAddr);
        buffer.append("/");
        buffer.append(dstPort);

        return buffer.toString();
    }

    /**
     * Handles a McastRtpPacket that belongs to this session. Verifies that the
     * sequence number is what was expected. Logs a fault if it doesn't match.
     * <p>
     * This is called directly by McastRtpDetect when it gets an appropriate
     * packet matching this session.
     * 
     * @param mcastRtpPacket
     *            - the packet that arrived
     * @see McastRtpDetect#packetArrived(net.sourceforge.jpcap.net.Packet)
     */
    public void mcastRtpPacketArrived(McastRtpPacket mcastRtpPacket) {

        /* warn on SSRC changes, but don't log a fault */
        if (mcastRtpPacket.getSsrc() != ssrc) {
            logger.warn("SSRC changed: was " + ssrc + ": now "
                    + mcastRtpPacket.getSsrc());
            logger.debug(mcastRtpPacket.toString());
        }

        /* raise an McastRtpFault if there is a sequence number gap */
        if (mcastRtpPacket.getSeq() != nextSeqExpected) {
            logger.warn("Sequence number error: expected "
                    + nextSeqExpected + ": got "
                    + mcastRtpPacket.getSeq());
            logger.debug(mcastRtpPacket.toString());

            McastRtpFault fault = new McastRtpFault(mcastRtpPacket);
            fault.setErrCode("1");
            fault.setErrMsg("Sequence number error");
            fault.setRuleFileName("MCAST_RTP_SEQ");
            fault.setExpected(nextSeqExpected);
            fault.setReceived(mcastRtpPacket.getSeq());
            fault.setGap(lastPacketTime,
                    mcastRtpPacket.getArrivalTime());
            addFault(fault);
        }
        ssrc = mcastRtpPacket.getSsrc();
        lastSeqReceived = mcastRtpPacket.getSeq();
        nextSeqExpected = (lastSeqReceived + 1) & 0xffff;
        lastPacketTime = mcastRtpPacket.getArrivalTime();
    }

    /**
     * Add a fault to the GUI and also to the multicast RTP fault history.
     * 
     * @param fault
     *            the fault to add
     */
    private void addFault(McastRtpFault fault) {

        /*
         * Add the fault to the diagnosis module
         */
        Activator.getDyswis().addFault(fault);

        /*
         * in a perfect world, this would be a call-back from the diagnosis
         * module
         */
        Activator.mcastRtpFaultListener.faultAdded(fault);
    }

    /**
     * @return the sessionStartTime
     */
    public long getSessionStartTime() {
        return sessionStartTime;
    }

    /**
     * @param sessionStartTime
     *            the sessionStartTime to set
     */
    public void setSessionStartTime(long sessionStartTime) {
        this.sessionStartTime = sessionStartTime;
    }

    /**
     * @return the lastPacketTime
     */
    public long getLastPacketTime() {
        return lastPacketTime;
    }

    /**
     * @param lastPacketTime
     *            the lastPacketTime to set
     */
    public void setLastPacketTime(long lastPacketTime) {
        this.lastPacketTime = lastPacketTime;
    }

}
