/**
 * COMS E6181: Advanced Internet Services (Fall 2011)
 * pgs2111 at columbia.edu
 */
package edu.columbia.cs.dyswis.module_mcast_rtp;

import java.util.HashMap;

import org.apache.log4j.Logger;

import edu.columbia.cs.dyswis.common.baseclass.DysProbe;
import edu.columbia.cs.dyswis.service.DyswisService;

/**
 * Probe to query the multicast RTP session history for a matching session.
 * <p>
 * A fault occurs during specific time range. It is useful to know whether or
 * not a remote node was joined to the same stream and the same time.
 * 
 * @author phil
 */
public class McastRtpSessionHistoryProbe implements DysProbe {

    /** for logging */
    private static Logger logger = Logger
            .getLogger("McastRtpSessionHistoryProbe");

    /**
     * Check the session history for the session
     * <p>
     * 
     * @param faultMap
     *            either an McastRtpFault, or a HashMap containing the
     *            SESSION_ID, GAP_START, and GAP_END
     * @return true if a matching session was found
     */
    @Override
    public Object probe(Object faultMap) {

        logger.debug("probe: " + faultMap);

        String sessionId;
        long gapStart;
        long gapEnd;

        if (faultMap instanceof HashMap<?, ?>) {
            HashMap<?, ?> map = (HashMap<?, ?>) faultMap;
            sessionId = (String) map.get("SESSIONID");
            gapStart = Long.parseLong((String) map.get("GAP_START"));
            gapEnd = Long.parseLong((String) map.get("GAP_END"));
        } else if (faultMap instanceof McastRtpFault) {
            McastRtpFault fault = (McastRtpFault) faultMap;
            sessionId = fault.getSessionId();
            gapStart = fault.getGapStart();
            gapEnd = fault.getGapEnd();
        } else {
            logger.debug("Unknown fault type: " + faultMap.toString());
            return null;
        }

        McastRtpSession session =
                Activator.mcastRtpDetect.getSession(sessionId);

        /*
         * If we don't have a matching session at all, return false
         */
        if (session == null) {
            logger.debug("No matching session found");
            return false;
        }
        /*
         * If the session started after the gap occurred, the information isn't
         * useful
         */
        if (session.getSessionStartTime() > gapStart) {
            logger.debug("Start time was after gap");
            return false;
        }

        /*
         * If we haven't received a packet after the gap ended, the information
         * isn't useful
         */
        if (session.getLastPacketTime() < gapEnd) {
            logger.debug("Session may have ended prior to gap");
            return false;
        }

        /*
         * We have a good match - the same session, encompassing the entire gap
         */
        logger.debug("Good session match!");
        return true;
    }

    /**
     * Display a result, if called directly by a LocalProbe or RemoteProbe.
     * <p>
     * Not really expecting this to be called .. this is handled through
     * MultiProbe now.
     */
    @Override
    public void displayResult(Object request, Object result) {

        DyswisService gui = Activator.getDyswis();
        gui.addDiagnosisMessageTitle("Multicast RTP Session History Probe");
        if ((Boolean) result) {
            gui.addDiagnosisMessageContent("Remote side found session",
                    0);
        } else {
            gui.addDiagnosisMessageContent(
                    "Remote side didn't find session", 0);
        }
    }

}
